"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class RuntimeArgumentsHandler {
    constructor() {
        this.registeredConfigs = new Set();
    }
    setProvider(provider) {
        this.provider = provider;
    }
    async getRuntimeArguments() {
        if (!this.provider) {
            throw new Error(`runtime arguments provider was not provided at runtime`);
        }
        const runtimeArguments = await this.provider();
        const runtimeConfigs = [...this.registeredConfigs.values()].flat();
        return {
            ...runtimeArguments,
            config: [...runtimeArguments.config, ...runtimeConfigs],
        };
    }
    registerRuntimeConfig(config) {
        this.registeredConfigs.add(config);
    }
    removeRuntimeConfig(config) {
        this.registeredConfigs.delete(config);
    }
}
exports.default = new RuntimeArgumentsHandler();
//# sourceMappingURL=runtime-arguments-provider.js.map