"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBaseCjsModuleSystem = void 0;
const global_this_js_1 = require("./global-this.js");
const falseModule = {
    get exports() {
        return {};
    },
    filename: '',
    id: '',
    children: [],
};
function createBaseCjsModuleSystem(options) {
    const { resolveFrom, dirname, readFileSync, globals = {}, loadModuleHook } = options;
    const requireCache = new Map();
    const load = loadModuleHook ? loadModuleHook(loadModule) : loadModule;
    return {
        requireModule(filePath) {
            var _a;
            if (filePath === false) {
                return {};
            }
            const fileModule = (_a = requireCache.get(filePath)) !== null && _a !== void 0 ? _a : load(filePath);
            return fileModule.exports;
        },
        requireFrom(contextPath, request) {
            return loadFrom(contextPath, request).exports;
        },
        resolveFrom,
        requireCache,
        globals,
    };
    function resolveThrow(contextPath, request, requestOrigin) {
        const resolvedRequest = resolveFrom(contextPath, request, requestOrigin);
        if (resolvedRequest === undefined) {
            throw new Error(`Cannot resolve "${request}" in ${requestOrigin || contextPath}`);
        }
        return resolvedRequest;
    }
    function loadFrom(contextPath, request, requestOrigin) {
        var _a;
        const existingRequestModule = requireCache.get(request);
        if (existingRequestModule) {
            return existingRequestModule;
        }
        const resolvedPath = resolveThrow(contextPath, request, requestOrigin);
        if (resolvedPath === false) {
            return falseModule;
        }
        return (_a = requireCache.get(resolvedPath)) !== null && _a !== void 0 ? _a : load(resolvedPath);
    }
    function loadModule(filePath) {
        const newModule = { exports: {}, filename: filePath, id: filePath, children: [] };
        const contextPath = dirname(filePath);
        const fileContents = readFileSync(filePath);
        if (filePath.endsWith('.json')) {
            newModule.exports = JSON.parse(fileContents);
            requireCache.set(filePath, newModule);
            return newModule;
        }
        const localRequire = (request) => {
            const childModule = loadFrom(contextPath, request, filePath);
            if (childModule !== falseModule && !newModule.children.includes(childModule)) {
                newModule.children.push(childModule);
            }
            return childModule.exports;
        };
        localRequire.resolve = (request) => resolveThrow(contextPath, request, filePath);
        const moduleBuiltins = {
            module: newModule,
            exports: newModule.exports,
            __filename: filePath,
            __dirname: contextPath,
            require: localRequire,
        };
        const injectedGlobals = {
            global: global_this_js_1.envGlobal,
            ...globals,
        };
        const fnArgs = Object.keys(moduleBuiltins).join(', ');
        const globalsArgs = Object.keys(injectedGlobals).join(', ');
        const moduleSource = `${fileContents}\n//# sourceURL=${filePath}\n`;
        const globalFn = eval(`(function (${globalsArgs}){ return (function (${fnArgs}){${moduleSource}}); })`);
        requireCache.set(filePath, newModule);
        try {
            const moduleFn = globalFn(...Object.values(injectedGlobals));
            moduleFn(...Object.values(moduleBuiltins));
        }
        catch (e) {
            requireCache.delete(filePath);
            // switch to Error.cause once more places support it
            if (e instanceof Error && !e.filePath) {
                e.filePath = filePath;
            }
            throw e;
        }
        return newModule;
    }
}
exports.createBaseCjsModuleSystem = createBaseCjsModuleSystem;
//# sourceMappingURL=base-cjs-module-system.js.map