"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SetMultiMap = void 0;
class SetMultiMap {
    constructor() {
        this.map = new Map();
    }
    get size() {
        return Array.from(this.map.values())
            .map(({ size }) => size)
            .reduce((sum, size) => sum + size, 0);
    }
    get(key) {
        return this.map.get(key);
    }
    add(key, value) {
        const valueSet = this.map.get(key);
        if (valueSet) {
            valueSet.add(value);
        }
        else {
            this.map.set(key, new Set([value]));
        }
        return this;
    }
    clear() {
        this.map.clear();
    }
    delete(key, value) {
        const valueSet = this.map.get(key);
        if (valueSet) {
            const wasInSet = valueSet.delete(value);
            if (valueSet.size === 0) {
                this.map.delete(key);
            }
            return wasInSet;
        }
        return false;
    }
    deleteKey(key) {
        return this.map.delete(key);
    }
    has(key, value) {
        const valueSet = this.map.get(key);
        return valueSet ? valueSet.has(value) : false;
    }
    hasKey(key) {
        const existingSet = this.map.get(key);
        return !!existingSet && existingSet.size > 0;
    }
    [Symbol.iterator]() {
        return this.entries();
    }
    *entries() {
        const { map } = this;
        for (const [key, valueSet] of map.entries()) {
            for (const value of valueSet) {
                yield [key, value];
            }
        }
    }
    *values() {
        const { map } = this;
        for (const valueSet of map.values()) {
            for (const value of valueSet) {
                yield value;
            }
        }
    }
    keys() {
        return this.map.keys();
    }
}
exports.SetMultiMap = SetMultiMap;
//# sourceMappingURL=set-multi-map.js.map