"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.hooks = exports.diagnostics = void 0;
const feature_1 = require("./feature");
const native_reserved_lists_1 = require("../native-reserved-lists");
const STCustomState = __importStar(require("./st-custom-state"));
const STCustomSelector = __importStar(require("./st-custom-selector"));
const diagnostics_1 = require("../diagnostics");
const selector_1 = require("../helpers/selector");
const is_vendor_prefixed_1 = __importDefault(require("is-vendor-prefixed"));
exports.diagnostics = {
    UNKNOWN_STATE_USAGE: (0, diagnostics_1.createDiagnosticReporter)('08001', 'error', (name) => `unknown pseudo-class "${name}"`),
};
// HOOKS
exports.hooks = (0, feature_1.createFeature)({
    transformSelectorNode({ context, selectorContext }) {
        const { currentAnchor, node, rule, scopeSelectorAst } = selectorContext;
        if (node.type !== 'pseudo_class') {
            return;
        }
        // find matching custom state
        let foundCustomState = false;
        for (const { symbol, meta } of currentAnchor.resolved) {
            // Handle node resolve mapping for custom-selector.
            // Currently custom selectors cannot get to this point in the process,
            // due to them being replaced at the beginning of the transform process.
            // However by using an internal process to analyze the context of selectors for
            // the language service, a source selector can reach this point without the initial
            // transform. This code keeps the custom selector untouched, but registers the AST it resolves to.
            // ToDo: in the future we want to move the custom selector transformation inline, or remove it all together.
            const customSelector = node.value.startsWith('--') &&
                symbol['-st-root'] &&
                STCustomSelector.getCustomSelectorExpended(meta, node.value.slice(2));
            if (customSelector) {
                const mappedSelectorAst = (0, selector_1.parseSelectorWithCache)(customSelector, { clone: true });
                const mappedContext = selectorContext.createNestedContext(mappedSelectorAst);
                // ToDo: wrap in :is() to get intersection of selectors
                scopeSelectorAst(mappedContext);
                if (mappedContext.currentAnchor) {
                    selectorContext.setNodeResolve(node, mappedContext.currentAnchor.resolved);
                }
                return; // this is not a state
            }
            //
            const states = symbol[`-st-states`];
            if (states && Object.hasOwnProperty.call(states, node.value)) {
                foundCustomState = true;
                // transform custom state
                if (selectorContext.transform) {
                    STCustomState.transformPseudoClassToCustomState(states, meta, node.value, node, meta.namespace, context.resolver, context.diagnostics, rule);
                }
                break;
            }
        }
        // handle nested pseudo classes
        if (node.nodes && !foundCustomState) {
            if (node.value === 'global') {
                // ignore `:st-global` since it is handled after the mixin transformation
                return;
            }
            else {
                // pickup all nested selectors except nth initial selector
                const innerSelectors = (node.nodes[0] && node.nodes[0].type === `nth` ? node.nodes.slice(1) : node.nodes);
                const nestedContext = selectorContext.createNestedContext(innerSelectors);
                scopeSelectorAst(nestedContext);
                /**
                 * ToDo: remove once elements is deprecated!
                 * support deprecated elements.
                 * used to flatten nested elements for some native pseudo classes.
                 */
                if (node.value.match(/not|any|-\w+?-any|matches|is|where|has|local/)) {
                    // delegate elements of first selector
                    selectorContext.elements[selectorContext.selectorIndex].push(...nestedContext.elements[0]);
                }
            }
        }
        // warn unknown state
        if (!foundCustomState &&
            !native_reserved_lists_1.nativePseudoClasses.includes(node.value) &&
            !(0, is_vendor_prefixed_1.default)(node.value) &&
            !selectorContext.isDuplicateStScopeDiagnostic()) {
            context.diagnostics.report(exports.diagnostics.UNKNOWN_STATE_USAGE(node.value), {
                node: rule,
                word: node.value,
            });
        }
    },
});
//# sourceMappingURL=css-pseudo-class.js.map