"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exec = exports.compileAsEntry = void 0;
const NativeModule = require('module');
const NodeTemplatePlugin = require('webpack/lib/node/NodeTemplatePlugin');
const NodeTargetPlugin = require('webpack/lib/node/NodeTargetPlugin');
const LibraryTemplatePlugin = require('webpack/lib/LibraryTemplatePlugin');
const EntryPlugin = require('webpack/lib/EntryPlugin');
const LimitChunkCountPlugin = require('webpack/lib/optimize/LimitChunkCountPlugin');
function compileAsEntry(compilation, context, request, plugins = []) {
    const pluginName = 'compileAsEntry';
    const outputOptions = {
        filename: '*',
    };
    const childCompiler = compilation.createChildCompiler(`${pluginName} ${request}`, outputOptions, []);
    new NodeTemplatePlugin(outputOptions).apply(childCompiler);
    new LibraryTemplatePlugin(null, 'commonjs2').apply(childCompiler);
    new NodeTargetPlugin().apply(childCompiler);
    new EntryPlugin(context, request, pluginName).apply(childCompiler);
    new LimitChunkCountPlugin({ maxChunks: 1 }).apply(childCompiler);
    plugins.forEach((p) => p.apply(childCompiler));
    let source;
    childCompiler.hooks.compilation.tap(pluginName, (compilation) => {
        compilation.hooks.afterProcessAssets.tap(pluginName, () => {
            source = compilation.assets['*'].source().toString();
            for (const chunk of compilation.chunks) {
                for (const file of chunk.files) {
                    compilation.deleteAsset(file);
                }
            }
        });
    });
    return new Promise((res, rej) => {
        childCompiler.runAsChild((err, _entries, compilation) => {
            var _a;
            if (err) {
                return rej(err);
            }
            if ((_a = compilation === null || compilation === void 0 ? void 0 : compilation.errors) === null || _a === void 0 ? void 0 : _a.length) {
                return rej(compilation.errors[0]);
            }
            if (!source) {
                throw new Error('child compiler has no source output');
            }
            res(source);
        });
    });
}
exports.compileAsEntry = compileAsEntry;
const exec = (code, filename, context) => {
    const module = new NativeModule(filename);
    module.paths = NativeModule._nodeModulePaths(context);
    module.filename = filename;
    module._compile(code, filename);
    return module.exports;
};
exports.exec = exec;
//# sourceMappingURL=compile-as-entry.js.map