"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectDependenciesDeep = exports.createContentHashPerMeta = exports.ensureHash = exports.rewriteImports = exports.createNamespaceMapping = exports.createMetadataForStylesheet = void 0;
const hash_content_util_1 = require("./hash-content-util");
function createMetadataForStylesheet(stylable, content, resourcePath, exposeNamespaceMapping = true, meta = stylable.fileProcessor.processContent(content, resourcePath)) {
    const usedMeta = collectDependenciesDeep(stylable, meta);
    const hashes = createContentHashPerMeta(usedMeta.keys());
    const stylesheetMapping = rewriteImports(usedMeta, hashes);
    const namespaceMapping = exposeNamespaceMapping ? createNamespaceMapping(usedMeta, hashes) : {};
    return {
        usedMeta,
        stylesheetMapping,
        namespaceMapping,
        entry: `/${ensureHash(meta, hashes)}.st.css`,
    };
}
exports.createMetadataForStylesheet = createMetadataForStylesheet;
function createNamespaceMapping(usedMeta, hashes) {
    const namespaceMapping = {};
    for (const [meta] of usedMeta) {
        namespaceMapping[`/${ensureHash(meta, hashes)}.st.css`] = meta.namespace;
    }
    return namespaceMapping;
}
exports.createNamespaceMapping = createNamespaceMapping;
function rewriteImports(usedMeta, hashes) {
    var _a;
    const sourcesByHash = {};
    for (const [meta, resolvedImports] of usedMeta.entries()) {
        const hash = ensureHash(meta, hashes);
        const rawAst = meta.sourceAst.clone();
        for (const { resolved, stImport } of resolvedImports) {
            if (resolved && resolved._kind === 'css') {
                const rawRule = (_a = rawAst.nodes) === null || _a === void 0 ? void 0 : _a.find(ruleByLocation(stImport.rule));
                if (!rawRule) {
                    throw new Error('Could not find source node for ' +
                        stImport.rule.toString() +
                        ' at ' +
                        meta.source);
                }
                if (rawRule.type === 'rule') {
                    rawRule.walkDecls((decl) => {
                        if (decl.prop === `-st-from`) {
                            decl.value = JSON.stringify(`/${ensureHash(resolved.meta, hashes)}.st.css`);
                        }
                    });
                }
                else if (rawRule.type === 'atrule') {
                    rawRule.params = rawRule.params.replace(stImport.request, `/${ensureHash(resolved.meta, hashes)}.st.css`);
                }
                else {
                    throw new Error('Unknown import rule ' + rawRule.toString());
                }
            }
            else if (resolved && resolved._kind === 'js') {
                throw new Error('js import are not supported yet!');
            }
        }
        sourcesByHash[`/${hash}.st.css`] = rawAst.toString();
    }
    return sourcesByHash;
}
exports.rewriteImports = rewriteImports;
function ruleByLocation(ruleA) {
    return (ruleB) => {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r;
        return (((_b = (_a = ruleB.source) === null || _a === void 0 ? void 0 : _a.start) === null || _b === void 0 ? void 0 : _b.column) === ((_d = (_c = ruleA.source) === null || _c === void 0 ? void 0 : _c.start) === null || _d === void 0 ? void 0 : _d.column) &&
            ((_f = (_e = ruleB.source) === null || _e === void 0 ? void 0 : _e.start) === null || _f === void 0 ? void 0 : _f.line) === ((_h = (_g = ruleA.source) === null || _g === void 0 ? void 0 : _g.start) === null || _h === void 0 ? void 0 : _h.line) &&
            ((_k = (_j = ruleB.source) === null || _j === void 0 ? void 0 : _j.end) === null || _k === void 0 ? void 0 : _k.column) === ((_m = (_l = ruleA.source) === null || _l === void 0 ? void 0 : _l.end) === null || _m === void 0 ? void 0 : _m.column) &&
            ((_p = (_o = ruleB.source) === null || _o === void 0 ? void 0 : _o.end) === null || _p === void 0 ? void 0 : _p.line) === ((_r = (_q = ruleA.source) === null || _q === void 0 ? void 0 : _q.end) === null || _r === void 0 ? void 0 : _r.line));
    };
}
function ensureHash(meta, hashes) {
    const hash = hashes.get(meta);
    if (!hash) {
        throw new Error('Missing meta hash for:' + meta.source);
    }
    return hash;
}
exports.ensureHash = ensureHash;
function createContentHashPerMeta(usedMeta) {
    const hashes = new Map();
    for (const meta of usedMeta) {
        hashes.set(meta, (0, hash_content_util_1.hashContent)(meta.sourceAst.toString()));
    }
    return hashes;
}
exports.createContentHashPerMeta = createContentHashPerMeta;
function collectDependenciesDeep(stylable, meta, out = new Map()) {
    if (out.has(meta)) {
        return out;
    }
    const imports = [];
    out.set(meta, imports);
    for (const stImport of meta.getImportStatements()) {
        const resolved = stylable.resolver.resolveImported(stImport, '');
        imports.push({ stImport, resolved });
        if (resolved && resolved._kind === 'css') {
            collectDependenciesDeep(stylable, resolved.meta, out);
        }
    }
    return out;
}
exports.collectDependenciesDeep = collectDependenciesDeep;
//# sourceMappingURL=create-metadata-stylesheet.js.map