"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HTMLSnapshotPlugin = void 0;
const path_1 = require("path");
const compile_as_entry_1 = require("./compile-as-entry");
const webpack_plugin_1 = require("@stylable/webpack-plugin");
class HTMLSnapshotPlugin {
    constructor(options) {
        this.outDir = options.outDir || '';
        this.render = options.render || (() => false);
        this.userGetLogicModule = options.getLogicModule;
    }
    apply(compiler) {
        compiler.hooks.thisCompilation.tap('HTMLSnapshotPlugin', (compilation) => {
            compilation.hooks.additionalAssets.tapPromise('HTMLSnapshotPlugin', async () => {
                const stylableModules = (0, webpack_plugin_1.uniqueFilterMap)(compilation.modules, (m) => {
                    return (0, webpack_plugin_1.isStylableModule)(m) ? m : null;
                });
                const getLogicModule = this.userGetLogicModule || (0, webpack_plugin_1.getCSSViewModuleWebpack)(compilation.moduleGraph);
                for (const module of stylableModules) {
                    await this.snapShotStylableModule(compilation, module, getLogicModule);
                }
            });
        });
    }
    async snapShotStylableModule(compilation, module, getLogicModule) {
        const component = getLogicModule(module, compilation.moduleGraph);
        if (!component || !component.context) {
            return;
        }
        const source = await (0, compile_as_entry_1.compileAsEntry)(compilation, component.context, component.resource);
        const componentModule = (0, compile_as_entry_1.exec)(source, component.resource, component.context);
        const html = this.render(componentModule, component);
        if (html === false) {
            return;
        }
        const targetPath = (0, path_1.join)(this.outDir, (0, path_1.basename)(component.resource)).replace(/\.[^.]+$/, '.snapshot.html');
        if (!compilation.assets[targetPath]) {
            compilation.assets[targetPath] = new compilation.compiler.webpack.sources.RawSource(html, false);
        }
        else {
            compilation.errors.push(new Error(`Duplicate component name ${component.resource} target path ${targetPath}`) // TODO: webpack types
            );
        }
    }
}
exports.HTMLSnapshotPlugin = HTMLSnapshotPlugin;
//# sourceMappingURL=stylable-html-snapshot.js.map