"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.metadataLoaderLocation = void 0;
const core_1 = require("@stylable/core");
const webpack_plugin_1 = require("@stylable/webpack-plugin");
const find_config_1 = __importDefault(require("find-config"));
const create_metadata_stylesheet_1 = require("./create-metadata-stylesheet");
let stylable;
const getLocalConfig = loadLocalConfigLoader();
const defaultOptions = {
    resolveNamespace: core_1.processNamespace,
    exposeNamespaceMapping: false,
};
exports.metadataLoaderLocation = __filename;
const metadataLoader = function (content) {
    const { resolveNamespace, exposeNamespaceMapping } = {
        ...defaultOptions,
        ...this.getOptions(),
        ...getLocalConfig(this.rootContext),
    };
    stylable = stylable || createStylable(this, resolveNamespace);
    const { usedMeta, ...output } = (0, create_metadata_stylesheet_1.createMetadataForStylesheet)(stylable, content, this.resourcePath, exposeNamespaceMapping);
    for (const [meta] of usedMeta) {
        this.addDependency(meta.source);
    }
    return 'export default ' + JSON.stringify(output);
};
function createStylable(loader, resolveNamespace) {
    if (!loader._compiler) {
        throw new Error('Stylable metadata loader requires a compiler instance');
    }
    return new core_1.Stylable({
        projectRoot: loader.rootContext,
        fileSystem: loader.fs,
        mode: loader._compiler.options.mode === 'development' ? 'development' : 'production',
        resolveOptions: {
            ...loader._compiler.options.resolve,
            extensions: [],
        },
        resolveNamespace,
        resolverCache: (0, webpack_plugin_1.createStylableResolverCacheMap)(loader._compiler),
    });
}
function loadLocalConfigLoader() {
    const localConfig = new Map();
    return (cwd) => {
        if (localConfig.has(cwd)) {
            return localConfig.get(cwd);
        }
        let config;
        try {
            config = find_config_1.default.require('stylable.config', { cwd }).metadataLoader;
        }
        catch (e) {
            config = {};
        }
        localConfig.set(cwd, config);
        return config;
    };
}
exports.default = metadataLoader;
//# sourceMappingURL=stylable-metadata-loader.js.map