"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IPCHost = exports.isParentProcess = void 0;
const engine_core_1 = require("@wixc3/engine-core");
const isParentProcess = (process) => {
    return process.constructor.name === 'process';
};
exports.isParentProcess = isParentProcess;
class IPCHost extends engine_core_1.BaseHost {
    constructor(process) {
        super();
        this.process = process;
        this.disposed = false;
        this.envs = new Set();
        this.onMessage = (message) => {
            this.emitMessageHandlers(message);
        };
        process.on('message', this.onMessage);
        process.once('disconnect', () => {
            process.off('message', this.onMessage);
            for (const env of this.envs) {
                this.emitMessageHandlers({
                    from: env,
                    type: 'dispose',
                    to: '*',
                    origin: env,
                    forwardingChain: [],
                });
            }
        });
    }
    postMessage(data) {
        this.envs.add(data.to);
        if (!this.process.send) {
            throw new Error('this process is not forked. There is not to send message to');
        }
        const disposeHandlers = (e) => {
            if (e) {
                this.emitMessageHandlers({
                    from: data.to,
                    type: 'dispose',
                    to: '*',
                    origin: data.to,
                    forwardingChain: [],
                });
            }
        };
        if ((0, exports.isParentProcess)(this.process)) {
            this.process.send(data, undefined, undefined, disposeHandlers);
        }
        else {
            this.process.send(data, disposeHandlers);
        }
    }
    dispose() {
        this.handlers.clear();
        this.process.removeAllListeners();
        this.disposed = true;
    }
    isDisposed() {
        return this.disposed;
    }
}
exports.IPCHost = IPCHost;
//# sourceMappingURL=ipc-host.js.map