"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WsServerHost = exports.WsHost = void 0;
const engine_core_1 = require("@wixc3/engine-core");
class WsHost extends engine_core_1.BaseHost {
    constructor(socket) {
        super();
        this.socket = socket;
        this.socket.on('message', (message) => {
            this.emitMessageHandlers(message);
        });
    }
    postMessage(data) {
        this.socket.emit('message', data);
    }
}
exports.WsHost = WsHost;
/**
 * TODO: handle disconnect
 */
class WsServerHost extends engine_core_1.BaseHost {
    constructor(server) {
        super();
        this.server = server;
        this.socketToEnvId = new Map();
        this.disposed = false;
        this.onConnection = (socket) => {
            const nameSpace = (original) => `${socket.id}/${original}`;
            const onMessage = (message) => {
                // this mapping should not be here because of forwarding of messages
                // maybe change message forwarding to have 'forward destination' and correct 'from'
                // also maybe we can put the init of the map on 'connection' event
                // maybe we can notify from client about the new connected id
                const originId = nameSpace(message.origin);
                const fromId = nameSpace(message.from);
                this.socketToEnvId.set(fromId, { socket, clientID: message.from });
                this.socketToEnvId.set(originId, { socket, clientID: message.origin });
                message.from = fromId;
                message.origin = originId;
                this.emitMessageHandlers(message);
            };
            socket.on('message', onMessage);
            socket.once('disconnect', () => {
                socket.off('message', onMessage);
                const disconnectedEnvs = [...this.socketToEnvId.entries()].reduce((acc, [envId, { socket: soc }]) => {
                    if (socket === soc) {
                        acc.add(envId);
                    }
                    return acc;
                }, new Set());
                for (const env of disconnectedEnvs) {
                    this.socketToEnvId.delete(env);
                    this.emitMessageHandlers({
                        type: 'dispose',
                        from: env,
                        origin: env,
                        to: '*',
                        forwardingChain: [],
                    });
                }
            });
        };
        this.server.on('connection', this.onConnection);
    }
    postMessage(data) {
        if (data.to !== '*') {
            if (this.socketToEnvId.has(data.to)) {
                const { socket, clientID } = this.socketToEnvId.get(data.to);
                data.to = clientID;
                socket.emit('message', data);
            }
            else {
                this.server.emit('message', data);
            }
        }
        else {
            this.server.emit('message', data);
        }
    }
    dispose() {
        this.handlers.clear();
        this.server.off('connection', this.onConnection);
        this.disposed = true;
    }
    isDisposed() {
        return this.disposed;
    }
}
exports.WsServerHost = WsServerHost;
//# sourceMappingURL=ws-node-host.js.map