"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.socketClientInitializer = void 0;
const ws_client_host_1 = require("../hosts/ws-client-host");
const socketClientInitializer = async ({ communication, env: { env }, ...socketClientOptions }) => {
    const url = communication.topology[env];
    if (!url) {
        throw new Error(`Could not find node topology for ${env} environment`);
    }
    const instanceId = env;
    const host = new ws_client_host_1.WsClientHost(url, socketClientOptions);
    if (communication.getEnvironmentHost(instanceId)) {
        communication.clearEnvironment(instanceId, undefined, false);
    }
    communication.registerMessageHandler(host);
    communication.registerEnv(instanceId, host);
    await host.connected;
    communication.handleReady({ from: instanceId });
    return {
        id: instanceId,
        onDisconnect: (cb) => {
            host.subscribers.on('disconnect', cb);
        },
        onReconnect: (cb) => {
            host.subscribers.on('reconnect', cb);
        },
    };
};
exports.socketClientInitializer = socketClientInitializer;
//# sourceMappingURL=socket-server.js.map