"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoggerService = void 0;
const patterns_1 = require("@wixc3/patterns");
const types_1 = require("../types");
class LoggerService extends patterns_1.EventEmitter {
    constructor(transports = [], baseMetadata = {}, config = { severity: types_1.LogLevel.DEBUG }) {
        super();
        this.transports = transports;
        this.baseMetadata = baseMetadata;
        this.config = config;
        this.messages = [];
        this.debug = (value) => this.log(value, types_1.LogLevel.DEBUG);
        this.info = (value) => this.log(value, types_1.LogLevel.INFO);
        this.warn = (value) => this.log(value, types_1.LogLevel.WARN);
        this.error = (value) => this.log(value, types_1.LogLevel.ERROR);
    }
    getMessages() {
        return this.messages;
    }
    logMessage(message) {
        if (message.level >= this.config.severity) {
            this.addToMessages(message);
            this.emit('message', message);
            if (this.config.logToConsole) {
                logToConsole(message);
            }
            for (const transport of this.transports) {
                transport.handleMessage(message);
            }
        }
    }
    clearMessages() {
        this.messages = [];
    }
    log(message, severity) {
        const { value, metadata } = getValue(message);
        this.logMessage({
            message: value,
            timestamp: Date.now(),
            level: severity,
            metadata: { base: this.baseMetadata, ...metadata },
        });
    }
    addToMessages(message) {
        if (this.messages.length + 1 > this.config.maxLogMessages) {
            this.messages.shift();
        }
        this.messages.push(message);
    }
}
exports.LoggerService = LoggerService;
function getValue(message) {
    const logValue = {
        value: '',
    };
    if (typeof message === 'function') {
        const evaluatedValue = message();
        if (typeof evaluatedValue === 'string') {
            logValue.value = evaluatedValue;
        }
        else {
            logValue.value = evaluatedValue.value;
            logValue.metadata = evaluatedValue.metadata || logValue.metadata;
        }
    }
    else if (typeof message === 'string') {
        logValue.value = message;
    }
    else {
        logValue.value = message.value;
        logValue.metadata = message.metadata;
    }
    return logValue;
}
function logToConsole({ message, metadata = {}, level }) {
    switch (level) {
        case types_1.LogLevel.DEBUG:
            console.log(message, metadata);
            break;
        case types_1.LogLevel.INFO:
            console.info(message, metadata);
            break;
        case types_1.LogLevel.WARN:
            console.warn(message, metadata);
            break;
        case types_1.LogLevel.ERROR:
            console.error(message, metadata);
            break;
    }
}
//# sourceMappingURL=logger-service.js.map