"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MapSlot = exports.MapRegistry = void 0;
const symbols_1 = require("../symbols");
const input_1 = require("./input");
class MapRegistry {
    constructor() {
        this.items = new Map();
        this.callbacks = new Set();
    }
    register(key, item) {
        this.items.set(key, item);
        for (const callback of this.callbacks) {
            callback(key, item);
        }
    }
    [Symbol.iterator]() {
        return this.items[Symbol.iterator]();
    }
    subscribe(cb) {
        this.callbacks.add(cb);
    }
    unSubscribe(cb) {
        this.callbacks.delete(cb);
    }
    stream(cb) {
        for (const [key, value] of this) {
            cb(key, value);
        }
        this.subscribe(cb);
        return () => this.unSubscribe(cb);
    }
    asMap() {
        return this.items;
    }
    get(key) {
        return this.items.get(key);
    }
    values() {
        return this.items.values();
    }
}
exports.MapRegistry = MapRegistry;
class MapSlot extends input_1.FeatureInput {
    static withType() {
        return {
            defineEntity(env) {
                return new MapSlot(env, env);
            },
        };
    }
    [symbols_1.CREATE_RUNTIME]() {
        return new MapRegistry();
    }
}
exports.MapSlot = MapSlot;
//# sourceMappingURL=map-slot.js.map