"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Slot = exports.Registry = void 0;
const symbols_1 = require("../symbols");
const input_1 = require("./input");
class Registry {
    constructor() {
        this.items = [];
        this.callbacks = new Set();
    }
    register(item) {
        this.items.push(item);
        for (const callback of this.callbacks) {
            callback(item);
        }
    }
    [Symbol.iterator]() {
        return this.items[Symbol.iterator]();
    }
    subscribe(cb) {
        this.callbacks.add(cb);
    }
    unSubscribe(cb) {
        this.callbacks.delete(cb);
    }
    asArray() {
        return this.items;
    }
    stream(cb) {
        for (const item of this) {
            cb(item);
        }
        this.subscribe(cb);
        return () => this.unSubscribe(cb);
    }
}
exports.Registry = Registry;
/**
 * Set of user typed values, consumed at any flow that requires this feature.
 *
 * @example
 * Slot.withType<IUser>().defineEntity('main')
 */
class Slot extends input_1.FeatureInput {
    /**
     * Provides the actual interface of each unit of the slot
     */
    static withType() {
        return {
            /**
             * Which env the slot will be assigned to.
             * @param env instance of the environment that the slot will be set upon
             */
            defineEntity(env) {
                return new Slot(env, env);
            },
        };
    }
    [symbols_1.CREATE_RUNTIME]() {
        return new Registry();
    }
}
exports.Slot = Slot;
//# sourceMappingURL=slot.js.map