"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpirableList = void 0;
/**
 * A list where items are automatically removed after a certain timeout.
 */
class ExpirableList {
    /**
     * @param expirationTimeMilliseconds the time to live for items in milliseconds
     */
    constructor(expirationTimeMilliseconds) {
        this.expirationTimeMilliseconds = expirationTimeMilliseconds;
        this.items = [];
    }
    /**
     * Adds new item to a collection
     */
    push(item) {
        this.removeExpiredItems();
        this.items.push({
            item,
            timestamp: performance.now(),
        });
    }
    /**
     * @returns the list of non-expired items
     */
    getItems() {
        this.removeExpiredItems();
        return this.items.map(({ item }) => item);
    }
    removeExpiredItems() {
        const currentTimestamp = performance.now();
        this.items = this.items.filter(({ timestamp }) => timestamp + this.expirationTimeMilliseconds > currentTimestamp);
    }
}
exports.ExpirableList = ExpirableList;
//# sourceMappingURL=expirable-list.js.map