"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerElectronHostEnvironment = exports.ElectronClientHost = void 0;
const engine_core_1 = require("@wixc3/engine-core");
/**
 * Communication host for the electron-renderer process
 */
class ElectronClientHost extends engine_core_1.BaseHost {
    constructor(host) {
        super();
        this.host = host;
        this.onMessage = (_, data) => {
            this.emitMessageHandlers(data);
        };
        this.host.on('message', this.onMessage);
    }
    postMessage(message) {
        this.host.send('message', message);
    }
    dispose() {
        this.host.off('message', this.onMessage);
    }
}
exports.ElectronClientHost = ElectronClientHost;
/**
 * register the ipcRenderer protocol as a host for envName
 * @param envName name of the environment to register
 * @param communication runtime instance of current environment communication
 */
function registerElectronHostEnvironment(ipcRenderer, envName, communication) {
    const comHost = new ElectronClientHost(ipcRenderer);
    communication.registerEnv(envName, new ElectronClientHost(ipcRenderer));
    communication.registerMessageHandler(comHost);
    return comHost;
}
exports.registerElectronHostEnvironment = registerElectronHostEnvironment;
//# sourceMappingURL=electron-client-host.js.map