"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveEnvironments = void 0;
const engine_core_1 = require("@wixc3/engine-core");
function resolveEnvironments(featureName, features, envTypes, filterByContext = true) {
    const environmentTypesToFilterBy = Array.isArray(envTypes) ? envTypes : [envTypes];
    const filteredEnvironments = new Set();
    const featureDefinition = features.get(featureName);
    if (!featureDefinition) {
        const featureNames = Array.from(features.keys());
        throw new Error(`cannot find feature ${featureName}. available features: ${featureNames.sort().join(', ')}`);
    }
    const { resolvedContexts } = featureDefinition;
    const deepDefsForFeature = (0, engine_core_1.flattenTree)(featureDefinition, (f) => f.dependencies.map((fName) => features.get(fName)));
    for (const { exportedEnvs = [] } of deepDefsForFeature) {
        for (const exportedEnv of exportedEnvs) {
            if ((!envTypes || environmentTypesToFilterBy.includes(exportedEnv.type)) &&
                (!filterByContext ||
                    !exportedEnv.childEnvName ||
                    resolvedContexts[exportedEnv.name] === exportedEnv.childEnvName)) {
                filteredEnvironments.add(exportedEnv);
            }
        }
    }
    return filteredEnvironments;
}
exports.resolveEnvironments = resolveEnvironments;
//# sourceMappingURL=environments.js.map