"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.launchEngineHttpServer = exports.DEFAULT_PORT = void 0;
const express_1 = __importDefault(require("express"));
const cors_1 = __importDefault(require("cors"));
const create_listening_server_1 = require("create-listening-server");
const socket_io_1 = __importDefault(require("socket.io"));
exports.DEFAULT_PORT = 3000;
const noContentHandler = (_req, res) => {
    res.status(204); // No Content
    res.end();
};
async function launchEngineHttpServer({ staticDirPath, httpServerPort = exports.DEFAULT_PORT, socketServerOptions, routeMiddlewares = [], } = {}) {
    const app = (0, express_1.default)();
    for (const { path, handlers } of routeMiddlewares) {
        app.use(path, handlers);
    }
    app.use((0, cors_1.default)());
    const { port, httpServer } = await (0, create_listening_server_1.safeListeningHttpServer)(httpServerPort, app);
    if (staticDirPath) {
        app.use('/', express_1.default.static(staticDirPath));
    }
    app.use('/favicon.ico', noContentHandler);
    const openSockets = new Set();
    httpServer.on('connection', (socket) => {
        openSockets.add(socket);
        socket.once('close', () => openSockets.delete(socket));
    });
    const socketServer = new socket_io_1.default.Server(httpServer, { cors: {}, ...socketServerOptions, transports: ['websocket'] });
    return {
        close: async () => {
            await new Promise((res, rej) => {
                for (const connection of openSockets) {
                    connection.destroy();
                }
                openSockets.clear();
                socketServer.close((e) => (e ? rej(e) : res()));
            });
        },
        port,
        app,
        socketServer,
    };
}
exports.launchEngineHttpServer = launchEngineHttpServer;
//# sourceMappingURL=launch-http-server.js.map