"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFeatureLoaders = exports.runNodeEnvironment = void 0;
const engine_core_1 = require("@wixc3/engine-core");
async function runNodeEnvironment({ featureName, childEnvName, features, bundlePath, config = [], name, type, options, host, env, }) {
    if (host) {
        config.push(engine_core_1.COM.use({
            config: {
                host,
                id: name,
            },
        }));
    }
    config.push(engine_core_1.RuntimeMetadata.use({
        engineerMetadataConfig: {
            applicationPath: bundlePath,
        },
    }));
    const featureLoaders = createFeatureLoaders(new Map(features), {
        name,
        childEnvName,
        type,
        env,
    });
    const rootFeatureLoader = featureLoaders[featureName];
    if (!rootFeatureLoader) {
        throw new Error("cannot find feature '" + featureName + "'. available features: " + Object.keys(featureLoaders).join(', '));
    }
    const { resolvedContexts = {} } = rootFeatureLoader;
    const featureLoader = new engine_core_1.FeatureLoadersRegistry(new Map(Object.entries(featureLoaders)), resolvedContexts);
    const optionsRecord = {};
    for (const [key, val] of options || []) {
        optionsRecord[key] = val;
    }
    const loadedFeatures = await featureLoader.getLoadedFeatures(featureName, optionsRecord);
    const runningFeatures = [loadedFeatures[loadedFeatures.length - 1]];
    const runtimeEngine = (0, engine_core_1.runEngineApp)({
        config,
        options: new Map(options),
        features: runningFeatures,
        resolvedContexts,
        env,
    });
    return runtimeEngine;
}
exports.runNodeEnvironment = runNodeEnvironment;
function createFeatureLoaders(features, { childEnvName, name: envName, env }) {
    const featureLoaders = {};
    for (const { scopedName, filePath, dependencies, envFilePaths, contextFilePaths, resolvedContexts, preloadFilePaths, } of features.values()) {
        featureLoaders[scopedName] = {
            preload: async (currentContext) => {
                var _a, _b;
                const initFunctions = [];
                if (childEnvName && currentContext[envName] === childEnvName) {
                    const contextPreloadFilePath = preloadFilePaths[`${envName}/${childEnvName}`];
                    if (contextPreloadFilePath) {
                        const preloadedContextModule = (await (_a = contextPreloadFilePath, Promise.resolve().then(() => __importStar(require(_a)))));
                        if (preloadedContextModule.init) {
                            initFunctions.push(preloadedContextModule.init);
                        }
                    }
                }
                const preloadFilePath = preloadFilePaths[envName];
                if (preloadFilePath) {
                    const preloadedModule = (await (_b = preloadFilePath, Promise.resolve().then(() => __importStar(require(_b)))));
                    if (preloadedModule.init) {
                        initFunctions.push(preloadedModule.init);
                    }
                }
                return initFunctions;
            },
            load: async (currentContext) => {
                var _a, _b, _c;
                if (childEnvName && currentContext[envName] === childEnvName) {
                    const contextFilePath = contextFilePaths[`${envName}/${childEnvName}`];
                    if (contextFilePath) {
                        await (_a = contextFilePath, Promise.resolve().then(() => __importStar(require(_a))));
                    }
                }
                for (const { env: envName } of new Set([env, ...env.dependencies])) {
                    const envFilePath = envFilePaths[envName];
                    if (envFilePath) {
                        await (_b = envFilePath, Promise.resolve().then(() => __importStar(require(_b))));
                    }
                }
                return (await (_c = filePath, Promise.resolve().then(() => __importStar(require(_c))))).default;
            },
            depFeatures: dependencies,
            resolvedContexts,
        };
    }
    return featureLoaders;
}
exports.createFeatureLoaders = createFeatureLoaders;
//# sourceMappingURL=node-environment.js.map