"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemoteNodeEnvironment = exports.startRemoteNodeEnvironment = void 0;
const child_process_1 = require("child_process");
const events_1 = require("events");
const forked_process_1 = require("./forked-process");
const types_1 = require("./types");
async function startRemoteNodeEnvironment(entryFilePath, { inspect, port, socketServerOptions = {}, requiredPaths = [] }) {
    const execArgv = inspect ? ['--inspect'] : [];
    const childProccess = (0, child_process_1.fork)(entryFilePath, [
        '--preferredPort',
        `${port}`,
        '--socketServerOptions',
        JSON.stringify(socketServerOptions),
        '--requiredPaths',
        JSON.stringify(requiredPaths),
    ], {
        execArgv,
    });
    await (0, events_1.once)(childProccess, 'message');
    childProccess.on('error', (e) => console.error(`error in forked process`, e));
    return {
        remoteNodeEnvironment: new RemoteNodeEnvironment(new forked_process_1.ForkedProcess(childProccess)),
        process: childProccess,
    };
}
exports.startRemoteNodeEnvironment = startRemoteNodeEnvironment;
class RemoteNodeEnvironment {
    constructor(childEnv) {
        this.childEnv = childEnv;
        this.messageHandlers = new Set();
    }
    async getRemotePort() {
        return new Promise((resolve) => {
            this.subscribe((message) => {
                if ((0, types_1.isEnvironmentPortMessage)(message)) {
                    resolve(message.payload.port);
                }
            });
            this.postMessage({ id: 'port-request' });
        });
    }
    subscribe(handler) {
        this.messageHandlers.add(handler);
        this.childEnv.on('message', handler);
    }
    unsubscribe(handler) {
        this.messageHandlers.delete(handler);
        this.childEnv.off('message', handler);
    }
    postMessage(message) {
        this.childEnv.postMessage(message);
    }
    dispose() {
        for (const handler of this.messageHandlers) {
            this.childEnv.off('message', handler);
        }
        this.messageHandlers.clear();
        if (this.childEnv && this.childEnv.terminate) {
            this.childEnv.terminate();
        }
    }
}
exports.RemoteNodeEnvironment = RemoteNodeEnvironment;
//# sourceMappingURL=remote-node-environment.js.map