"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("./constants");
const is_safari_1 = require("./is-safari");
const isMainThreadInSafari = is_safari_1.isSafari && !!globalThis.Worker;
(function () {
    if (!isMainThreadInSafari) {
        return;
    }
    const openWorkers = new Map();
    class WorkerBridgeMain extends (globalThis.Worker || {}) {
        constructor(url, options) {
            super(url, options);
            this.hookEnvironment();
        }
        hookEnvironment() {
            this.addEventListener('message', (e) => {
                const params = e.data;
                if ((params === null || params === void 0 ? void 0 : params.type) === constants_1.CREATE_NESTED_WORKER) {
                    const port = e.ports[0];
                    /**
                     * e.ports[0] === e.data.port // true
                     */
                    if (!port) {
                        throw new Error('no port');
                    }
                    if (openWorkers.has(port)) {
                        throw new Error('port already in use');
                    }
                    const args = params.constructorArgs;
                    const worker = new Worker(args[0], args[1]);
                    openWorkers.set(port, worker);
                    connectToWorker(worker, port);
                }
            });
        }
    }
    function connectToWorker(worker, port) {
        worker.addEventListener('message', (e) => {
            port.postMessage(e.data, e.ports.length ? [e.ports[0]] : undefined);
        });
        port.addEventListener('message', (e) => {
            worker.postMessage(e.data);
        });
        port.addEventListener('close', () => {
            worker.terminate();
            openWorkers.delete(port);
        });
        port.start();
    }
    globalThis.Worker = WorkerBridgeMain;
})();
//# sourceMappingURL=main-worker-host.js.map