"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("./constants");
class WorkerBridge {
    constructor(url, options) {
        /**
         * Listeners registered by the creator of the worker
         */
        this.messageListeners = new Set();
        this.errorListeners = new Set();
        this.channel = new MessageChannel();
        this.onmessage = null;
        this.onerror = null;
        this.onmessageerror = null;
        const messageToWorkerCreator = (e) => {
            for (const listener of this.messageListeners) {
                listener(e);
            }
        };
        this.channel.port1.onmessage = messageToWorkerCreator;
        const transferables = [this.channel.port2];
        // always sent to main thread
        self.postMessage({
            type: constants_1.CREATE_NESTED_WORKER,
            constructorArgs: [url, options],
            transferables,
        }, 
        /**
         * signature: postMessage(message: any, transfer?: Transferable[]): void;
         * https://developer.mozilla.org/en-US/docs/Web/API/DedicatedWorkerGlobalScope/postMessage
         */
        transferables);
    }
    /**
     * Send messages FROM the creator of the worker TO the nested worker
     * @param message any serializable object
     */
    postMessage(message) {
        this.channel.port1.postMessage(message);
    }
    /**
     * add listen from the worker creator ON the nested worker
     */
    addEventListener(type, listener) {
        if (type === 'message') {
            this.messageListeners.add(listener);
        }
        else if (type === 'error') {
            this.errorListeners.add(listener);
        }
        else {
            throw new Error('not supported');
        }
    }
    removeEventListener(type, listener) {
        if (type === 'message') {
            this.messageListeners.delete(listener);
        }
        else {
            throw new Error('not supported');
        }
    }
    terminate() {
        this.channel.port1.close();
        this.channel.port2.close();
    }
    dispatchEvent() {
        throw new Error('not supported');
    }
}
if (!globalThis.Worker) {
    globalThis.Worker = WorkerBridge;
}
//# sourceMappingURL=nested-worker-polyfill.js.map