"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDisposables = void 0;
/**
 * Disposables allow adding of disposal async functions,
 * when dispose is called, these functions will be run sequentially
 */
function createDisposables() {
    const disposables = new Set();
    return {
        async dispose() {
            const toDispose = Array.from(disposables).reverse();
            disposables.clear();
            for (const dispose of toDispose) {
                if (typeof dispose === 'function') {
                    await dispose();
                }
                else {
                    await dispose.dispose();
                }
            }
        },
        add(disposable) {
            disposables.add(disposable);
        },
        remove(disposable) {
            disposables.delete(disposable);
        },
    };
}
exports.createDisposables = createDisposables;
//# sourceMappingURL=create-disposables.js.map