"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LRUCache = void 0;
/**
 * BASIC (not optimal) implementation of the LRU cache
 */
class LRUCache {
    constructor(config = {}) {
        var _a;
        this.config = config;
        this.cache = new Map();
        this.keys = [];
        this.maxSize = (_a = config.maxSize) !== null && _a !== void 0 ? _a : Infinity;
        if (this.maxSize < 1) {
            throw new Error('LRUCache max size must be larger than 0');
        }
    }
    set(key, value) {
        this.updateKeysOrder(key);
        if (this.keys.length > this.maxSize) {
            const removedKey = this.keys.shift();
            this.cache.delete(removedKey);
        }
        this.cache.set(key, value);
    }
    get(key) {
        if (!this.cache.has(key)) {
            return;
        }
        this.updateKeysOrder(key);
        return this.cache.get(key);
    }
    delete(key) {
        const keyIndex = this.keys.indexOf(key);
        if (keyIndex > -1) {
            this.keys.splice(keyIndex, 1);
        }
        this.cache.delete(key);
    }
    has(key) {
        return this.cache.has(key);
    }
    size() {
        return this.cache.size;
    }
    clear() {
        this.cache.clear();
        this.keys = [];
    }
    updateKeysOrder(key) {
        const keyIndex = this.keys.indexOf(key);
        if (keyIndex > -1) {
            this.keys.splice(keyIndex, 1);
        }
        this.keys.push(key);
    }
}
exports.LRUCache = LRUCache;
//# sourceMappingURL=lru-cache.js.map