"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRoot = exports.defaultLayerSettings = exports.overlapBindConfig = void 0;
const core_1 = require("@zeejs/core");
const layout_overlay_1 = require("./layout-overlay");
const utils_1 = require("./utils");
exports.overlapBindConfig = Symbol(`overlap-bind`);
exports.defaultLayerSettings = {
    overlap: `window`,
    backdrop: `none`,
    generateElement: true,
};
function createRoot({ onChange, } = {}) {
    let idCounter = 0;
    const rootLayer = (0, core_1.createLayer)({
        extendLayer: {
            element: null,
            settings: exports.defaultLayerSettings,
            setElement: function (element) {
                if (this.element) {
                    throw new Error(`cannot setElement on a layer that was already initiated with an element`);
                }
                if (element.tagName !== `ZEEJS-LAYER`) {
                    throw new Error(`cannot setElement on a layer that is not <zeejs-layer>`);
                }
                this.element = element;
                initLayerElement(this);
                onChange === null || onChange === void 0 ? void 0 : onChange(`create`, this);
            },
        },
        defaultSettings: exports.defaultLayerSettings,
        onChange(reason, layer) {
            onChange === null || onChange === void 0 ? void 0 : onChange(reason, layer);
        },
        init(layer, settings) {
            layer.state = {
                mouseInside: false,
                focusInside: false,
                lastFocusedElement: null,
            };
            layer.settings = settings;
            layer.id = `zeejs-layer-${idCounter++}`;
            if (settings.generateElement && utils_1.isBrowser) {
                layer.element = document.createElement(`zeejs-layer`); // ToDo: test that each layer has a unique element
                initLayerElement(layer);
            }
        },
        destroy(layer) {
            if (layer[exports.overlapBindConfig]) {
                layer[exports.overlapBindConfig].stop(); // not tested because its a side effect:/
            }
        },
    });
    function initLayerElement(layer) {
        const { element, settings, parentLayer } = layer;
        element.id = layer.id;
        if (parentLayer) {
            if (settings.overlap === `window`) {
                element.classList.add(`zeejs--overlapWindow`);
            }
            else if (settings.overlap instanceof HTMLElement) {
                element.classList.add(`zeejs--overlapElement`);
                layer[exports.overlapBindConfig] = (0, layout_overlay_1.layoutOverlay)(settings.overlap, element);
            }
        }
    }
    return rootLayer;
}
exports.createRoot = createRoot;
//# sourceMappingURL=root.js.map