"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createListeningHttpServer = void 0;
const node_http_1 = require("node:http");
const node_events_1 = require("node:events");
/**
 * Creates a listening `http.Server` instance using a specific `port`.
 * Resolves when the server is listening, or rejects if an error occurs.
 *
 * @param port the port to try listening to
 * @param requestListener optional request listener. can also be added later with `httpServer.on('request', cb)`
 * @param hostname optional host to pass to .listen()
 */
async function createListeningHttpServer(port, requestListener, hostname) {
    const httpServer = (0, node_http_1.createServer)(requestListener);
    httpServer.listen(port, hostname);
    await (0, node_events_1.once)(httpServer, "listening");
    return httpServer;
}
exports.createListeningHttpServer = createListeningHttpServer;
//# sourceMappingURL=create-listening-server.js.map