"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTempDirectorySync = exports.createTempDirectory = void 0;
const node_os_1 = require("node:os");
const node_path_1 = require("node:path");
const node_fs_1 = require("node:fs");
/**
 * Create an empty, unique directory in the current OS's temp directory.
 *
 * @param prefix optional prefix to add to the random name. (default: "temp-")
 * @returns an absolute `path` and a `remove()` function.
 */
async function createTempDirectory(prefix = "temp-") {
    const path = await node_fs_1.promises.realpath(await node_fs_1.promises.mkdtemp((0, node_path_1.join)((0, node_os_1.tmpdir)(), prefix)));
    const remove = () => node_fs_1.promises.rm(path, { recursive: true, force: true });
    return { path, remove };
}
exports.createTempDirectory = createTempDirectory;
/**
 * Create an empty, unique directory in the current OS's temp directory.
 *
 * @param prefix optional prefix to add to the random name. (default: "temp-")
 * @returns an absolute `path` and a `remove()` function.
 */
function createTempDirectorySync(prefix = "temp-") {
    const path = node_fs_1.realpathSync.native((0, node_fs_1.mkdtempSync)((0, node_path_1.join)((0, node_os_1.tmpdir)(), prefix)));
    const remove = () => (0, node_fs_1.rmSync)(path, { recursive: true, force: true });
    return { path, remove };
}
exports.createTempDirectorySync = createTempDirectorySync;
//# sourceMappingURL=create-temp-directory.js.map