"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getElementSize = exports.getElementDimensions = exports.getDimensionsFromRect = exports.getSizeFromDimensions = exports.observeWindowDimensions = exports.observeElementDimensions = exports.unmeasuredDimensions = void 0;
exports.unmeasuredDimensions = {
    width: null,
    height: null,
};
const observeElementDimensions = (element, callback) => {
    const observer = new ResizeObserver(() => callback(element.getBoundingClientRect()));
    callback(element.getBoundingClientRect());
    observer.observe(element);
    return () => observer.disconnect();
};
exports.observeElementDimensions = observeElementDimensions;
const observeWindowDimensions = (callback) => {
    const onResize = () => callback({
        width: window.innerWidth,
        height: window.innerHeight,
    });
    onResize();
    window.addEventListener('resize', onResize, {
        capture: false,
        passive: true,
    });
    return () => window.removeEventListener('resize', onResize);
};
exports.observeWindowDimensions = observeWindowDimensions;
const getSizeFromDimensions = (dimensions, sizeAsHeight) => {
    var _a;
    return (_a = (sizeAsHeight ? dimensions.height : dimensions.width)) !== null && _a !== void 0 ? _a : 0;
};
exports.getSizeFromDimensions = getSizeFromDimensions;
const getDimensionsFromRect = (rect) => {
    return rect
        ? {
            width: rect.width,
            height: rect.height,
        }
        : exports.unmeasuredDimensions;
};
exports.getDimensionsFromRect = getDimensionsFromRect;
const getElementDimensions = (element) => {
    return element
        ? (0, exports.getDimensionsFromRect)(element.getBoundingClientRect())
        : {
            width: window.innerWidth,
            height: window.innerHeight,
        };
};
exports.getElementDimensions = getElementDimensions;
/**
 * Get Element or Window size
 *
 * @param {HTMLElement | null | undefined} element if Element isn't provided, Window size would be returned
 * @param {boolean} sizeAsHeight for vertical list size is height; for horizontal list size is width
 * @return {number} size in pixels
 */
const getElementSize = (element, sizeAsHeight) => {
    return (0, exports.getSizeFromDimensions)(element
        ? (0, exports.getElementDimensions)(element)
        : {
            width: window.innerWidth,
            height: window.innerHeight,
        }, sizeAsHeight);
};
exports.getElementSize = getElementSize;
//# sourceMappingURL=dimensions.js.map