"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useDelayedUpdateState = exports.unchanged = exports.useDelayedUpdate = void 0;
const react_1 = require("react");
const useDelayedUpdate = () => {
    const [_, update] = (0, react_1.useReducer)((n) => ++n, 0);
    const triggeredForUpdate = (0, react_1.useRef)({ handle: null });
    (0, react_1.useEffect)(() => {
        return () => {
            // eslint-disable-next-line react-hooks/exhaustive-deps
            const handle = triggeredForUpdate.current.handle;
            if (handle !== null) {
                window.cancelAnimationFrame(handle);
            }
        };
    }, []);
    return (0, react_1.useCallback)(() => {
        if (triggeredForUpdate.current.handle !== null) {
            return;
        }
        const cb = () => {
            triggeredForUpdate.current.handle = null;
            update();
        };
        triggeredForUpdate.current.handle = window.requestAnimationFrame(cb);
    }, []);
};
exports.useDelayedUpdate = useDelayedUpdate;
exports.unchanged = Symbol('unchanged');
const useDelayedUpdateState = (setValue) => {
    const triggeredForUpdate = (0, react_1.useRef)({ handle: null });
    (0, react_1.useEffect)(() => {
        return () => {
            // eslint-disable-next-line react-hooks/exhaustive-deps
            const handle = triggeredForUpdate.current.handle;
            if (handle !== null) {
                window.cancelAnimationFrame(handle);
            }
        };
    }, []);
    return (0, react_1.useCallback)((value) => {
        if (triggeredForUpdate.current.handle !== null) {
            return;
        }
        const cb = () => {
            triggeredForUpdate.current.handle = null;
            const val = typeof value === 'function' ? value() : value;
            if (val !== exports.unchanged) {
                setValue(val);
            }
        };
        triggeredForUpdate.current.handle = window.requestAnimationFrame(cb);
    }, [setValue]);
};
exports.useDelayedUpdateState = useDelayedUpdateState;
//# sourceMappingURL=use-delayed-update.js.map