"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useIdBasedKeyboardNav = void 0;
const react_1 = require("react");
const common_1 = require("../common");
const useIdBasedKeyboardNav = (elementsParent, focusedId, setFocusedId, selectedId, setSelectedId) => {
    const onKeyPress = (0, react_1.useCallback)((ev) => {
        if (!elementsParent.current) {
            return;
        }
        const element = elementsParent.current;
        const children = (0, common_1.childrenById)(element);
        if (!focusedId) {
            return;
        }
        const currentFocused = children[focusedId];
        if (!currentFocused) {
            return;
        }
        const gotoDir = (dir) => {
            const focusedRect = currentFocused.getBoundingClientRect();
            let nextId = null;
            let nextIdDistance = Number.MAX_SAFE_INTEGER;
            let nextElement = null;
            const focusMiddle = dir === 'up'
                ? (focusedRect.top + focusedRect.height / 2) * -1
                : dir === 'down'
                    ? focusedRect.top + focusedRect.height / 2
                    : dir === 'left'
                        ? (focusedRect.left + focusedRect.width / 2) * -1
                        : focusedRect.left + focusedRect.width / 2;
            const focusLeft = dir === 'up'
                ? focusedRect.left
                : dir === 'down'
                    ? focusedRect.left
                    : dir === 'left'
                        ? focusedRect.top
                        : focusedRect.top;
            const focusRight = dir === 'up'
                ? focusedRect.right
                : dir === 'down'
                    ? focusedRect.right
                    : dir === 'left'
                        ? focusedRect.bottom
                        : focusedRect.bottom;
            for (const [id, element] of Object.entries(children)) {
                if (id !== focusedId) {
                    const rect = element.getBoundingClientRect();
                    const rectStart = dir === 'up'
                        ? rect.bottom * -1
                        : dir === 'down'
                            ? rect.top
                            : dir === 'left'
                                ? rect.right * -1
                                : rect.left;
                    const rectLeft = dir === 'up'
                        ? rect.left
                        : dir === 'down'
                            ? rect.left
                            : dir === 'left'
                                ? rect.top
                                : rect.top;
                    const rectRight = dir === 'up'
                        ? rect.right
                        : dir === 'down'
                            ? rect.right
                            : dir === 'left'
                                ? rect.bottom
                                : rect.bottom;
                    if (rectStart > focusMiddle) {
                        const distanceMain = rectStart - focusMiddle;
                        const distanceSides = Math.max(Math.max(rectLeft - focusLeft, 0), Math.max(focusRight - rectRight, 0), 0);
                        const distance = Math.sqrt(Math.pow(distanceMain, 2) + Math.pow(distanceSides, 2));
                        if (distance < nextIdDistance) {
                            nextId = id;
                            nextIdDistance = distance;
                            nextElement = element;
                        }
                    }
                }
            }
            if (nextId) {
                setFocusedId(nextId);
                if (nextElement) {
                    nextElement.scrollIntoView({
                        behavior: 'smooth',
                        inline: 'nearest',
                        block: 'nearest',
                    });
                }
            }
        };
        switch (ev.code) {
            case common_1.KeyCodes.ArrowLeft: {
                gotoDir('left');
                break;
            }
            case common_1.KeyCodes.ArrowRight:
                gotoDir('right');
                break;
            case common_1.KeyCodes.ArrowUp:
                gotoDir('up');
                break;
            case common_1.KeyCodes.ArrowDown:
                gotoDir('down');
                break;
            case common_1.KeyCodes.Space:
            case common_1.KeyCodes.Enter:
                setSelectedId(focusedId);
                break;
            default:
        }
    }, [elementsParent, focusedId, setFocusedId, setSelectedId]);
    return onKeyPress;
};
exports.useIdBasedKeyboardNav = useIdBasedKeyboardNav;
//# sourceMappingURL=use-keyboard-nav.js.map